﻿using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Lims.Extensions;
using Kivii.Linq;
using Kivii.Text;
using Kivii.Third.Bestseller.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Third.Bestseller.Entities
{
    public class BestsellerOrderBase : IEntityIsThirdReportOrder
    {
        private JsonServiceClient _client = null;

        public Report OnInstanceThirdReport(Guid ThirdReportOrderKvid)
        {
            return new Report();
        }

        public object OnPullThirdReport()
        {
            //throw new Exception("请使用三六一专用拉取订单接口!");
            #region 设置缓存1分钟内不能连续请求
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey($"{typeof(BestsellerOrder).FullName}.PullOrder:LockedOneMinute");
            var cacheValue = cache.Get<string>(cacheKey);
            if (!cacheValue.IsNullOrEmpty())
            {
                throw new Exception(cacheValue);
            }
            #endregion

            var rtns = new RestfulQueryResponse<ThirdReportOrder>();

            var client = new JsonHttpClient(Configs._defaultUrl);
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(BestsellerOrder).FullName);
            //DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime));
            var response = client.PullOrders(Configs._token, Configs._secret, maxTranslateTime);
            cache.Set(cacheKey, "一分钟内不可多次查询!", TimeSpan.FromMinutes(1));

            (response.code != 200).ThrowIfTrue(response.message);
            rtns.Results = new List<ThirdReportOrder>();
            if (response.data == null) return rtns;
            if (response.data.results.IsNullOrEmpty()) return rtns;

            var results = response.data.results.AcceptThirdContractOrder(Guid.Parse(Configs._ownerKvid), Configs._ownerName, conn);
            foreach (var item in results)
            {
                client.AcceptOrderConfirm(Configs._token, Configs._secret, item.SerialNumber);
            }
            rtns.Results.AddRange(results);
            rtns.Total = rtns.Results.Count;

            return rtns;
        }

        public bool OnRejectThirdReport(Guid ThirdReportOrderKvid)
        {
            return false;
        }

        public bool OnUploadThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(BestsellerOrder).FullName) return false;
            //订单还未受理的 则自动去找相关报告进行关联
            if (!third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return true;
            var thirdDetails = conn.Select<ThirdReportOrderDetail>(o => o.ThirdOrderKvid == third.Kvid);
            if (thirdDetails.IsNullOrEmpty()) return false;
            var reports = conn.SelectByIds<Report>(thirdDetails.ConvertAll(o => o.ReportKvid));
            if (reports.IsNullOrEmpty()) return false;
            if (reports.Exists(o => o.Status != (int)ReportStatusType.ReportCollected)) return false;

            var client = new JsonHttpClient(Configs._defaultUrl);

            var orderResult = new OrderResult();
            orderResult.ORDER_NO = third.SerialNumber;
            orderResult.RESULT = "4";
            orderResult.RPT_ITEMS = new List<ReportDetail>();

            var orderPdfs = new List<OrderPdf>();

            //针对多个报告存在不合格的项目 则回传订单整体判定为不合格
            var infoJudement = true;

            foreach (var thirdDetail in thirdDetails)
            {
                if (thirdDetail.ReportKvid == Guid.Empty) continue;
                var report = reports.FirstOrDefault(o => o.Kvid == thirdDetail.ReportKvid);
                if (report == null) continue;
                if (report.Status != (int)ReportStatusType.ReportCollected) continue;
                var queryReportItems = conn.From<ReportItem>();
                queryReportItems.Where(o => o.ReportKvid == report.Kvid);
                var allReportItems = conn.Select(queryReportItems);
                var primaryAllReportItems = allReportItems.filterReportItem();
                primaryAllReportItems.ForEach(o => o.PackageUnit = string.Empty);
                var selfItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();
                var partItems = allReportItems.Where(o => o.Type == ReportItemType.SamplePart).ToList();
                //合并色号和部位
                if (!selfItems.IsNullOrEmpty())
                {
                    foreach (var item in selfItems)
                    {
                        var primarys = primaryAllReportItems.Where(o => o.RootKvid == item.RootKvid).ToList();
                        foreach (var primary in primarys)
                        {
                            primary.PackageUnit = string.Empty;
                            primary.PackageUnit = item.Title;
                        }
                    }
                }
                if (!partItems.IsNullOrEmpty())
                {
                    foreach (var item in partItems)
                    {
                        var primarys = primaryAllReportItems.Where(o => o.ParentKvid == item.Kvid).ToList();
                        foreach (var primary in primarys)
                        {
                            primary.TitlePart = string.Empty;
                            primary.TitlePart = item.Title;
                        }
                    }
                }

                if (report.TestJudgement == "合格") orderResult.RESULT = "1";
                if (report.TestJudgement == "不合格")
                {
                    infoJudement = false;
                    orderResult.RESULT = "2";
                }

                #region 报告PDF
                var code = report.Kvid.ToString().Substring(0, 4);
                var internalCode = $"{code}/{report.ReportId}";

                var orderPdf = new OrderPdf();
                orderPdf.RPT_NO = report.ReportId;
                orderPdf.URL = $"{Configs._pdfUrl}/Lims/Report/{internalCode}";
                orderPdfs.Add(orderPdf);
                #endregion

                #region 报告及报告项目
                var rptItem = new ReportDetail();
                rptItem.RPT_NO = report.ReportId;
                rptItem.RPT_RESULT = "4";
                if (report.TestJudgement == "合格") rptItem.RPT_RESULT = "1";
                if (report.TestJudgement == "不合格")
                {
                    infoJudement = false;
                    rptItem.RPT_RESULT = "2";
                }
                rptItem.ITEMS = new List<Detail>();
                foreach (var item in primaryAllReportItems)
                {
                    var nodes = allReportItems.Where(o => o.ParentKvid == item.Kvid).ToList();
                    var detail = new Detail();
                    detail.F1_DISPLAY = item.PackageUnit;
                    detail.CHECK_PART_DISPLAY = item.TitlePart;
                    detail.ITEM_NAME = item.Title;
                    detail.ITEM_STANDARD = item.DetectionCodeActual;
                    detail.RESULT = "4";
                    if (item.Judgement == "合格") detail.RESULT = "1";
                    if (item.Judgement == "不合格")
                    {
                        infoJudement = false;
                        detail.RESULT = "2";
                    }
                    List<string> qualifiedValue = new List<string>();
                    qualifiedValue.Add(item.DetectionValueQualified);
                    List<string> testValue = new List<string>();
                    testValue.Add(item.TestValue);
                    if (!nodes.IsNullOrEmpty())
                    {
                        foreach (var node in nodes)
                        {
                            qualifiedValue.Add($"{node.Title} {node.DetectionValueQualified}");
                            testValue.Add($"{node.Title} {node.TestValue}");
                        }
                    }
                    qualifiedValue.RemoveAll(o => o.IsNullOrEmpty());
                    testValue.RemoveAll(o => o.IsNullOrEmpty());
                    detail.ITEM_SPEC = string.Join(",", qualifiedValue);
                    detail.RESULT_REMARK = string.Join(",", testValue);
                    rptItem.ITEMS.Add(detail);
                }
                orderResult.RPT_ITEMS.Add(rptItem);
                #endregion
            }

            //针对多个报告存在不合格的项目 则回传订单整体判定为不合格
            if (!infoJudement) orderResult.RESULT = "2";

            var respResult = client.UploadOrderResult(Configs._token, Configs._secret, orderResult);

            if (respResult.code == 200)
            {
                var respPdf = client.UploadOrderPdf(Configs._token, Configs._secret, orderResult.ORDER_NO, orderPdfs);
                if (respPdf.code == 200)
                {
                    var complete = client.UploadOrderComplete(Configs._token, Configs._secret, orderResult);
                    third.IsUploaded = true;
                    third.AddOnlyProperties(o => o.IsUploaded);
                    conn.UpdateOnly(third);

                    var log = new EntityLog<ThirdReportOrder>();
                    log.OwnerKvid = third.Kvid;
                    log.BizId = third.SerialNumber;
                    log.BizKvid = third.Kvid;
                    log.BizType = third.ThirdType;
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Remark = $"Code:{complete.code}  Message:{complete.message}";
                    conn.Insert(log);
                    return true;
                }
                else return false;
            }
            else return false;
        }
    }
}
